/*
 *  cli.h
 *
 */

#ifndef __CLI_H_
#define __CLI_H_

#define CLI_MAGIC		0x82371143
#define CLI_RESPONSE		0x77315328
#define MAX_USER_MBX_SIZE	2048
#define MAX_CMD_LINE		256
#define HISTORY_SIZE		10

typedef struct CmdDebug {
	u_long		tm_init;
	u_long		total_cmds;
	u_long		total_users;
} CmdDebug_t;

typedef struct CmdEntry {
	u_long		flags;
	char		*cmd;
	char		*usage;
	char		*desc;
	int		(*handler)(int, char **);
	struct CmdEntry	*next;
	struct CmdEntry	*prev;
} CmdEntry_t;


typedef struct CmdUser {
	int		state;
	char		*who;
	int		in_fd;
	int		out_fd;
	char		in_buf[MAX_USER_MBX_SIZE];
	char		out_buf[MAX_USER_MBX_SIZE];
	char		history[HISTORY_SIZE][MAX_CMD_LINE];
	int		in_len;
	int		out_len;
	int		echo_ptr;
	int		echo_on;
	int		history_ptr;
	int		history_used;
	int		arrow_ptr;
	int		tabs;
	int		(*str_handler)(void *, char *);
	int		(*output_handler)(u_char, struct CmdUser *, void *, int);
	int		(*message_handler)(struct CmdUser *);
	u_char		output_flags;
	void		*str_arg;

	struct termio	old_termio;

	struct sockaddr_in raddr;

	struct CmdUser	*next;
	struct CmdUser	*prev;
} CmdUser_t;


#define USER_STATE_AUTH		0
#define USER_STATE_CMD		1
#define USER_STATE_STR		2
#define USER_STATE_MESSAGE	3
#define USER_STATE_PASSTHRU	4

#define CMD_FLAG_HIDDEN		0x00000001
#define CMD_FLAG_MESSAGE	0x00000002

#define FIND_BY_EITHER		0x00
#define FIND_BY_IN		0x01
#define FIND_BY_OUT		0x02
#define FIND_BY_BOTH		(FIND_BY_IN | FIND_BY_OUT)

#define ECHO_OUTPUT		0x01
#define RESPONSE_OUTPUT		0x02
#define PROMPT_OUTPUT		0x04
#define MESSAGE_OUTPUT		0x08

extern CmdEntry_t *	cli_find_cmd(char *cmd);
extern CmdEntry_t *	cli_find_message_cmd(char *cmd);
extern CmdEntry_t *	cli_find_partial_cmd(char *cmd, int tabs);
extern CmdUser_t *	cli_find_user(int in_fd, int out_fd, int type);
extern void		cli_fd_event(int fd, void *arg, int r, int w, int e);
extern void		cli_set_debug(int debug, int debugfd);
extern void		cli_set_prompt(char *new_prompt);
extern void		cli_reprint_prompt(CmdUser_t *up);
extern int		cli_receive_data(CmdUser_t *up);
extern int		cli_init(u_short debug_port);
extern int		cli_add_cmd(char *cmd, char *usage, char*desc,
					int (*handler)(int, char **), u_long flags);
extern int		cli_remove_cmd(char *cmd);
extern int		cli_add_console(char *cmd,
			     int (*output_handler)(u_char, struct CmdUser *, void *, int));
extern int		cli_add_user(int in_fd, int out_fd, char *who,
					struct sockaddr_in *raddr, int prompt,
			     int (*output_handler)(u_char, struct CmdUser *, void *, int));
extern int		cli_remove_user(int in_fd, int out_fd, int type);
extern int		cli_start_user(CmdUser_t *up, int prompt);
extern int		cli_output(CmdUser_t *up, char *buf, int len, u_char type);
extern int		cli_flush(CmdUser_t *up);

/* From netkit-ftp - All credit goes to them for these functions */
extern char *		slurpstring(void);
extern char **		nmakeargv(char *line, int *pargc, char **parg);
extern int		another(char *line, int *pargc, char ***pargv, const char *prompt);
extern int		cli_getstr(CmdUser_t *up, int (*str_handler)(void *, char *),
									   void *str_arg);
extern int 		cli_printf(char *fmt, ...);

extern CmdUser_t	*CmdUserList;
extern CmdUser_t	*CmdUserList_end;
extern CmdUser_t	*current_user;

#endif /* __CLI_H_ */
