/*
 *  sched.h
 *
 */

#ifndef __SCHED_H_
#define __SCHED_H_

#define SCHED_MAX_FD_NAME	128

typedef struct SchedTime
{
	long		secs;
	long		usecs;
} SchedTime_t;

typedef struct SchedDebug {
	u_long		time_sched;
	u_long		last_sched;
	u_long		last_ran;
	u_long		times_ran;
	u_long		reads;
	u_long		writes;
	u_long		exceptions;
	SchedTime_t	last_time;
	SchedTime_t	total_time;
} SchedDebug_t;

typedef struct SchedTask {
	u_long		tid;
	u_long		flags;
	u_long		time_val;
	u_char		*task_name;
	void		*arg;
	void		(*task_ptr)(void *);
	SchedDebug_t	debug;
	int		reschedule;

	struct SchedTask *next;
	struct SchedTask *prev;
} SchedTask_t;

typedef struct SchedFD {
	int		fd;
	char		name[SCHED_MAX_FD_NAME];
	u_long		flags;
	void		*arg;
	void		(*handler)(int, void *, int, int, int);
	SchedDebug_t	debug;

	struct SchedFD	*next;
	struct SchedFD	*prev;
} SchedFD_t;

#define TASK_FLAG_CLEANUP	0x00000001
#define TASK_FLAG_IMMEDIATE	0x00000002
#define TASK_FLAG_RESCHEDULE	0x00000004
#define TASK_FLAG_PRIORITY	0x00000008
#define TASK_FLAG_DONE		0x00000010
#define TASK_FLAG_RESCHED_ONCE	0x00000020

#define FD_FLAG_READ		0x00000001
#define FD_FLAG_WRITE		0x00000002
#define FD_FLAG_EXCEPT		0x00000004

#define MATCH_PTR		0x00000001
#define MATCH_ARG		0x00000002
#define MATCH_NAME		0x00000004

extern SchedTask_t *	sched_find_task(void (*task_ptr)(void *), void *arg,
							char *name, u_long match);
extern SchedTask_t *	sched_find_done_task(void (*task_ptr)(void *), void *arg,
							char *name, u_long match);
extern SchedTask_t *	sched_find_task_by_tid(int tid);
extern SchedFD_t *	sched_find_fd(int fd);
extern void		sched_shutdown(void) ;
extern void		sched_stop_loop(void) ;
extern void		sched_set_debug(int debug, int debug_fd);
extern void		sched_move_to_end(SchedTask_t *task);
extern void		sched_loop(void);
extern void		sched_register_cmds(int (*add_cmd)(char *, char *, char *,
					int (*handler)(int, char **), u_long),
						int (*print_func)(char *fmt, ...)) ;
extern int		sched_init(void);
extern int		sched_cleanup( char *name, void (*func_ptr)(void *), void *arg );
extern int		sched_task(void (*task_ptr)(void *), void *arg, u_long flags,
					 u_long time_val, char *name);
extern int		sched_remove_task(void (*task_ptr)(void *), void *arg,
							char *name, u_long match);
extern int		sched_reschedule(u_long time_val);
extern int		sched_add_fd(int fd, void (*handler)(int, void *, int, int, int),
				void *arg, u_long flags, char *name);
extern int		sched_fd_save(int fd, SchedFD_t *to);
extern int		sched_fd_restore(int fd, SchedFD_t *from);
extern int		sched_remove_fd(int fd);
extern int		sched_fd_flags(int fd, u_long flags);
extern int		sched_stop(void);

extern SchedTask_t	*current;
extern SchedFD_t	*current_fd;

#endif /* __SCHED_H_ */
