/*
 *  typedefs.h
 *
 */

#ifndef __TYPEDEFS_H__
#define __TYPEDEFS_H__

/*==============================
 *	Sizes
 *==============================*/
typedef unsigned long	ulong_t;	/* 32 bit    */
typedef unsigned int	uint_t;		/* 32 bit    */
typedef unsigned short  ushort_t;	/* 16 bit    */
typedef unsigned char   uchar_t; 	/*  8 bit    */

/*==============================
 *	addresses
 *==============================*/
typedef unsigned long   paddr_t;	/* kernel - physical address */
typedef unsigned long	vaddr_t;	/* kernel - virtual address  */
typedef unsigned long	uaddr_t;	/* user   - virtual address */


/*================================================================
 *	Byte swapping and Memory alignment	
 *===============================================================*/
#define BYTE_SWAP(w) (((w)&0xff)<<24)      | \
                     (((w)&0xff00)<<8)     | \
		     (((w)&0xff0000)>>8)   | \
		     (((w)&0xff000000)>>24)

#define NIBLE_SWAP(b)   ( ((b >> 4) & 0x0f) | ((b << 4) & 0xf0))
#define MEM_ALIGN(m, v) ( ((ulong_t)m + (v-1)) & ~(v-1)) 

/*=============================
 *	Input/Output
 *=============================*/

#define Out32(addr, v)		*(volatile ulong_t *)(addr)=(ulong_t)v
#define Out16(addr, v)		*(volatile ushort_t *)(addr)=(ushort_t)v
#define Out8(addr, v)	 	*(volatile uchar_t *)(addr)=(uchar_t)v

#define Inp32(addr)		*(volatile ulong_t *)(addr)
#define Inp16(addr)		*(volatile ushort_t *)(addr)
#define Inp8(addr)		*(volatile uchar_t *)(addr)

#define GT_Read(addr, p) \
         *p = *(volatile ulong_t *)(addr); \
	 *p = BYTE_SWAP(*p)

#define GT_Write(addr, v) \
	*(volatile ulong_t *)(addr)=BYTE_SWAP((ulong_t)v)


/*==========================
 *	Time Constants
 *==========================*/
#define MEGA_HZ			133
#define TICKS_PER_SECOND	(MEGA_HZ*1024*1024) 
#define ONE_SECOND		(MEGA_HZ*1024*1024)
#define ONE_MILI_SECOND		(ONE_SECOND)/1000
#define ONE_MICRO_SECOND	(ONE_SECOND)/1000000

/*======================================
 * 	Disable/Enable Interrupts
 *======================================*/
#define disable_intr(x)         save_flags(x); cli()
#define enable_intr(x)          restore_flags(x)

/*=====================
 *	Bits
 *=====================*/
#define BIT(bit)	(1 << bit)
#define BITS(val,bits)	((ulong_t)val << bits)

/*====================================
 *    Little Endian bits
 *    31-24  23-16  15-08  07-00
 *====================================*/
#define NO_BIT          0x00000000
#define BIT0            0x00000001
#define BIT1            0x00000002
#define BIT2            0x00000004
#define BIT3            0x00000008
#define BIT4            0x00000010
#define BIT5            0x00000020
#define BIT6            0x00000040
#define BIT7            0x00000080
#define BIT8            0x00000100
#define BIT9            0x00000200
#define BIT10           0x00000400
#define BIT11           0x00000800
#define BIT12           0x00001000
#define BIT13           0x00002000
#define BIT14           0x00004000
#define BIT15           0x00008000
#define BIT16           0x00010000
#define BIT17           0x00020000
#define BIT18           0x00040000
#define BIT19           0x00080000
#define BIT20           0x00100000
#define BIT21           0x00200000
#define BIT22           0x00400000
#define BIT23           0x00800000
#define BIT24           0x01000000
#define BIT25           0x02000000
#define BIT26           0x04000000
#define BIT27           0x08000000
#define BIT28           0x10000000
#define BIT29           0x20000000
#define BIT30           0x40000000
#define BIT31           0x80000000

/*=========================
 *	Memory sizes
 *=========================*/
#define _1K             0x00000400
#define _2K             0x00000800
#define _4K             0x00001000
#define _8K             0x00002000
#define _16K            0x00004000
#define _32K            0x00008000
#define _64K            0x00010000
#define _128K           0x00020000
#define _256K           0x00040000
#define _512K           0x00080000

#define _1M             0x00100000
#define _2M             0x00200000
#define _3M             0x00300000
#define _4M             0x00400000
#define _5M             0x00500000
#define _6M             0x00600000
#define _7M             0x00700000
#define _8M             0x00800000
#define _9M             0x00900000
#define _10M            0x00a00000
#define _11M            0x00b00000
#define _12M            0x00c00000
#define _13M            0x00d00000
#define _14M            0x00e00000
#define _15M            0x00f00000
#define _16M            0x01000000
#define _24M		0x01800000
#define _32M            0x02000000
#define _48M		0x03000000
#define _64M            0x04000000
#define _128M           0x08000000
#define _256M		0x10000000
#define _512M		0x20000000
#define _1G		0x40000000

#endif /* __TYPEDEFS_H__ */
